#include "WPILib.h"
#include "WeejusControls.h"
#include "WeejusDrive.h"
#include "WeejusAuto.h"

class thunderWeejus: public SimpleRobot {
public:
	WeejusControls *_controls;
	WeejusDrive *_drive;
	WeejusAuto *_auto;
	WeejusClimber *_climber;
	WeejusShooter *_shooter;

	thunderWeejus(void) {
		_drive = WeejusDrive::getInstance();
		_controls = WeejusControls::getInstance();
		_auto = WeejusAuto::getInstance();
		_climber = WeejusClimber::getInstance();
		_shooter = WeejusShooter::getInstance();
		// increase analog sampling rate to maximum possible for 1 analog module
		// ((500,000 / 8) is max for 5 msec loop)
		AnalogModule::GetInstance(1)->SetSampleRate(60000.0f);
	}

	void RobotInit() {
		printf("\n     ***ROBOT INITIALIZED***\n\n");
	}

	void Disabled(void) {
		printf("\n       ***DISABLED***\n\n");

		_controls->forceEnableLights(true);
		while (IsDisabled()) {
			_controls->process(WeejusControls::DISABLED);
			Wait(0.005);
		}
		_controls->forceEnableLights(false);
	}

	void Autonomous(void) {
		int mode;
		int delay;
		bool runAnyMode;

		printf("\n       ***AUTONOMOUS***\n\n");

		_controls->getAutoValues(&mode, &delay);
		printf("Autonoumous initialize other classes\n");
		_drive->initialize();
		_shooter->initialize(true);
		_climber->initialize();
		runAnyMode = _auto->initialize(mode, delay);
		while (IsAutonomous() && IsEnabled()) {
			if (runAnyMode) {
				_controls->process(WeejusControls::AUTONOMOUS);
				_auto->process();
				_shooter->process();
				_drive->process();
			}
			Wait(0.005); // wait for a motor update time
		}
		printf("Autonoumous loop ended\n");
		_auto->endAuto(true);
	}

	void OperatorControl(void) {
		printf("\n       ***TELEOP***\n\n");

		_drive->initialize();
		_climber->initialize();
		_shooter->initialize(false);
		while (IsOperatorControl() && IsEnabled()) {
			//weejus
			//sneff
			//wheeeeeeeerabouts
			_controls->process(WeejusControls::TELEOP);
			_drive->process();
			_climber->process();
			_shooter->process();
			Wait(0.005); // wait for a motor update time
		}
	}

	/**
	 * Runs during test mode
	 */
	void Test() {

	}
};

START_ROBOT_CLASS(thunderWeejus)
;

