#include "WeejusAuto.h"

#define USE_GYRO 0
#define USE_ANGLE_FUDGE 0

//#define ONE_FIRST_DRIVE 50			
//The driving to the cooperation bridge in position one in inches
//#define AUTO_DRIVE_SPEED  .4			
//The speed at which we drive in any direction


#define ERROR_MARGIN 25 				
//The number of degrees of error to allow when driving

WeejusAuto* WeejusAuto::_theAuto = 0;

WeejusAuto* WeejusAuto::getInstance() {
	if (_theAuto == NULL)
		_theAuto = new WeejusAuto();

	return _theAuto;
}

WeejusAuto::WeejusAuto() {
	_drive = WeejusDrive::getInstance();
	_shooter = WeejusShooter::getInstance();

	_autoStateMain = WeejusAuto::autoEnded;
	_autoStateMode1 = WeejusAuto::mode1Ended;
	//_autoStateMode2 = WeejusAuto::mode2Ended;
	// NOTE if you add to the above, also add to endAuto()
}

WeejusAuto::~WeejusAuto() {
	delete _theAuto;
}

/*
 void WeejusAuto::updateAngle(float desiredAngle)
 {
 if (!USE_GYRO)
 {
 return;
 }
 
 float currentAngle = _drive->getGyro();

 float newAngle = currentAngle + desiredAngle;
 if (newAngle >= currentAngle)
 {
 minAngle = currentAngle - ERROR_MARGIN;
 maxAngle = newAngle + ERROR_MARGIN;
 }
 else
 {
 minAngle = newAngle - ERROR_MARGIN;
 maxAngle = currentAngle + ERROR_MARGIN;
 }
 }*/
/*
 bool WeejusAuto::checkCourse()
 {
 if (!USE_GYRO)
 {
 return true;
 }
 
 float currentAngle = _drive->getGyro();
 if ((currentAngle >= minAngle) && (currentAngle <= maxAngle))
 {
 return true;
 }
 else
 {
 printf("off course\n");
 endAuto(false);
 return false;
 }
 }*/

// initialize autonomous (call it once)
bool WeejusAuto::initialize(int mode, int delay) {
	printf("mode:%d   delay:%d\n", mode, delay);

	_mode = mode;
	_delay = (float) delay;

	// if mode is zero (or not supported), then do nothing
	switch (_mode) {
	default:
	case 0:
		printf("Auto::initialize: do nothing (mode=%d)\n", _mode);
		endAuto(true);
		return (false);

	case 1:
	case 2:
	case 3:
		break;
	}

	// if there is no delay, then start the mode
	if (delay == 0) {
		// set state to the one after the delay
		_autoStateMain = WeejusAuto::autoRun;
		printf("Auto::initialize: no delay no\n");
	} else {
		//set to delay state
		_autoStateMain = WeejusAuto::autoDelay;
		printf("Auto::initialize: with delay\n");
	}

	// do the first loop to get things started
	process();
	return (true);
}

// process all auto steps (call it for every loop in autonomous)
bool WeejusAuto::process() {
	bool done;

	switch (_autoStateMain) {
	case WeejusAuto::autoDelay: {
		begin_time = GetTime();
		_autoStateMain = WeejusAuto::autoDelayCheck;
		printf("Auto::process: Waiting *autoDelay*\n");
		break;
	}

	case WeejusAuto::autoDelayCheck: {
		double curTime = GetTime();
		if (curTime >= (begin_time + (double) _delay)) {
			_autoStateMain = WeejusAuto::autoRun;
			printf("Auto::process: Delay is done *autoDelayCheck*\n");
		}
		break;
	}

	case WeejusAuto::autoRun: {
		switch (_mode) {
		default:
			printf("init default mode %d\n", _mode);
			endAuto(false);
			break;
		case 1:
			printf("init mode1\n");
			_autoSpeedMode1 = MODE1_SHOOT_POWER;
			_autoAngleMode1 = MODE1_SHOOT_ANGLE;
			_autoStateMode1 = WeejusAuto::mode1Lower;
			runMode1(); // prime the pump
			break;
		case 3:
			printf("init mode3\n");
			_autoSpeedMode1 = MODE3_SHOOT_POWER;
			_autoAngleMode1 = MODE3_SHOOT_ANGLE;
			_autoStateMode1 = WeejusAuto::mode1Lower;
			runMode1(); // prime the pump
			break;
		case 2:
			printf("init mode2\n");
			_autoSpeedMode1 = MODE2_SHOOT_POWER;
			_autoAngleMode1 = MODE2_SHOOT_ANGLE;
			_autoStateMode1 = WeejusAuto::mode1Lower;
			runMode1(); // prime the pump				
			//_autoStateMode2 = WeejusAuto::mode2Drive;
			//runMode2();
			break;
		}
		_autoStateMain = WeejusAuto::autoRunCheck;
		printf("Auto::process: initializing state\n");
		break;
	}

	case WeejusAuto::autoRunCheck: {
		// run appropriate state machine
		switch (_mode) {
		default:
			printf(
					"Auto::process: make sure you add the mode to this switch DUMMY!\n");
			done = true;
			break;
		case 1: // add here if more just stand place modes
		case 2:
		case 3:
			done = runMode1();
			break;
			//case 2:
			//	done = runMode2();
			//	break;
		}

		// when done running, then move shooter to feed position
		if (done) {
#if 0
			_autoStateMain = WeejusAuto::autoFeed;
			printf("Auto::process:moving shooter to feed position \n");
#else
			printf("Auto::process:autoRunCheck is done\n");
			endAuto(true);
			return (true);
#endif
		}
		break;
	}

	case WeejusAuto::autoFeed: {
		// move to feed angle
		_shooter->moveToAngle(ANGLE_FEED);
		printf("Auto::process:moving to feed position \n");
		endAuto(true);
		return (true);
	}

	case WeejusAuto::autoEnded: {
		// NO PRINTF
		return (true);
	}
	}
	// make sure we call the classes we are using
	//_drive->process();
	//_shooter->process();
	return (false);
}

bool WeejusAuto::runMode1() {
	switch (_autoStateMode1) {
	case WeejusAuto::mode1Lower:
		_shooter->autoWheelOn(true);
		_shooter->setSpeed(_autoSpeedMode1);
		_shooter->moveToAngle(_autoAngleMode1);
		_autoStateMode1 = WeejusAuto::mode1LowerCheck;
		printf("runMode1, going to angle\n");
		break;

	case WeejusAuto::mode1LowerCheck:
		if (_shooter->getIsAtAngle()) {
			_autoStateMode1 = WeejusAuto::mode1Shoot;
# if USE_ANGLE_FUDGE
			if (_mode == 1)
			{
				_discToLower = _shooter->getNumDiscs() - 1;
			}
#endif
			printf("runMode1, is at angle\n");
		}
		break;

	case WeejusAuto::mode1Shoot:
		_shooter->shootAll();
		_autoStateMode1 = WeejusAuto::mode1ShootCheck;
		printf("runMode1, shooting all discs\n");
		break;

	case WeejusAuto::mode1ShootCheck:
# if USE_ANGLE_FUDGE
		if (_mode == 1)
		{
			if (_discToLower == _shooter->getNumDiscs()) {
				_discToLower = 99;// so we don't do this again
				_shooter->moveToAngle(_autoAngleMode1 + .10);
				printf("Lower it!\n");
			} /*else if ((_discToLower - 1) == _shooter->getNumDiscs()) {
			 _shooter->moveToAngle(_autoAngleMode1 + .20);
			 printf("Lower it (again!)!\n");
			 _discToLower = -1;
			 }*/
		}
#endif
		if (_shooter->shootDone()) {
			_autoStateMode1 = WeejusAuto::mode1Ended;
			printf("runMode1, all discs have been shot,also zack is great\n");
			return (true);
		}
		break;

	case WeejusAuto::mode1Ended:
		return (true);
	}
	return (false);
}

/*
 bool WeejusAuto::runMode2()
 {
 switch(_autoStateMode2)
 {
 case WeejusAuto::mode2Drive:
 printf("runMode2, moving the robot to destination\n");
 _drive->autonomousMove(MODE2_DRIVE_DISTANCE, MODE2_DRIVE_MAX_POWER);
 _shooter->autoWheelOn(true);
 _shooter->setSpeed(MODE2_SHOOT_POWER);
 _shooter->moveToAngle(MODE2_SHOOT_ANGLE);
 _autoStateMode2 = WeejusAuto::mode2DriveCheck;
 break;
 
 case WeejusAuto::mode2DriveCheck:
 if (_drive->doneYet())
 {
 printf ("runMode2, done moving and started turning\n");
 _drive->autonomousTurn(MODE2_DRIVE_TURN);
 _autoStateMode2 = WeejusAuto::mode2TurnCheck;
 }	
 break;
 
 case WeejusAuto::mode2TurnCheck:
 if (_drive->doneYet() && (_shooter->getIsAtAngle()))
 {
 printf ("runMode2, done turning, started shooting\n");
 _shooter->shootAll();
 _autoStateMode2 = WeejusAuto::mode2ShootCheck;
 }		
 break;
 
 case WeejusAuto::mode2ShootCheck:
 if (_shooter->shootDone())
 {
 printf ("runMode2, done shooting\n");
 _autoStateMode2 = WeejusAuto::mode2Ended;
 return (true);
 }
 break;
 
 case WeejusAuto::mode2Ended:
 return (true);
 }
 return (false);
 }*/

void WeejusAuto::endAuto(bool endedOK) {
	_autoStateMain = WeejusAuto::autoEnded;
	_autoStateMode1 = WeejusAuto::mode1Ended;
	//_autoStateMode2 = WeejusAuto::mode2Ended;
	if (endedOK) {
		printf("endAuto: We are done\n");
	} else {
		printf("endAuto: An error has occured\n");
	}

	_drive->stopAction();
	_shooter->autoWheelOn(false);
	_shooter->stopShoot();
}
