#include "WeejusCamera.h"
#include "IOmap.h"
#include "nivision.h"

#define CLIMBER_SERVO_MIN 0.5f
#define CLIMBER_SERVO_MAX 1.0f
#define SHOOTER_SERVO_MIN 0.0f
#define SHOOTER_SERVO_MAX 0.5f

WeejusCamera::WeejusCamera() :
#if !NO_CAMERA
			// set the ACam& to the reference
			// from getInstance
			m_camera(AxisCamera::GetInstance(/*no arg: default cam IP*/)),
#endif
			m_tiltSrv(PWM_CAMERA_TILT_SERVO),
			m_lightRing(RELAY_CAMERA_RING_LIGHT, Relay::kForwardOnly),

			m_globalFlipped(false) {
#if !NO_CAMERA
	m_camera.WriteMaxFPS(MAX_FPS);
	m_camera.WriteResolution(RESOLUTION);
	m_camera.WriteBrightness(BRIGHTNESS);
	m_camera.WriteColorLevel(COLOR_LEVEL);
	m_camera.WriteExposureControl(EXPOSURE_CONTROL);
	m_camera.WriteExposurePriority(EXPOSURE_PRIORITY);
	m_camera.WriteCompression(COMPRESSION);
	m_camera.WriteWhiteBalance(WHITE_BALANCE);
	m_camera.WriteRotation(AxisCameraParams::kRotation_180);
#endif
	// probably not needed; see PWM.h for detais
	// m_tiltSrv.SetBounds(max, 0, 0, 0, min)
	m_servoShooter = 0.0f;
	m_servoClimber = CLIMBER_SERVO_MAX;
	m_servo = m_servoShooter;
	m_tiltSrv.Set(m_servo);
}

WeejusCamera::~WeejusCamera() {
	// nothing to flush
}

void WeejusCamera::setLight(bool on) {
	if (on) {
		//printf("light on\n");
		m_lightRing.Set(Relay::kOn);// both channels on
	} else {
		//printf("light off\n");
		m_lightRing.Set(Relay::kOff);// both channels off
	}
}

void WeejusCamera::flip(bool flipped) {
	// 180 degree rotation is the normal (unflipped) state
	m_globalFlipped = flipped;
	if (m_globalFlipped) {
		// climber is now front
		m_servo = m_servoClimber;
		localTilt(0.0f, CLIMBER_SERVO_MIN, CLIMBER_SERVO_MAX);
#if !NO_CAMERA
		m_camera.WriteRotation(AxisCameraParams::kRotation_0);
#endif
	} else {
		// shooter is now front
		m_servo = m_servoShooter;
		localTilt(0.0f, SHOOTER_SERVO_MIN, SHOOTER_SERVO_MAX);
#if !NO_CAMERA
		m_camera.WriteRotation(AxisCameraParams::kRotation_180);
#endif
	}
}

// Positive servo is up
void WeejusCamera::tiltUp(float amount) {
	if (m_globalFlipped) {
		// climber is the front
		m_servoClimber
				= localTilt(amount, CLIMBER_SERVO_MIN, CLIMBER_SERVO_MAX);
	} else {
		// shooter is the front
		m_servoShooter = localTilt(-amount, SHOOTER_SERVO_MIN,
				SHOOTER_SERVO_MAX);
	}
}

void WeejusCamera::tiltDown(float amount) {
	if (m_globalFlipped) {
		// climber is the front
		m_servoClimber = localTilt(-amount, CLIMBER_SERVO_MIN,
				CLIMBER_SERVO_MAX);
	} else {
		// shooter is the front
		m_servoShooter
				= localTilt(amount, SHOOTER_SERVO_MIN, SHOOTER_SERVO_MAX);
	}
}

float WeejusCamera::localTilt(float deg, float min, float max) {
	// 0 to 0.5 is shooter; 0 is all the way up
	// 0.5 to 1 is climber; 1 is all the way up

	// convert degrees to servo units AND cal new setting
	m_servo += (deg / SERVO_RANGE);

	// stay within given raw value range
	if (m_servo < min) {
		m_servo = min;
	} else if (m_servo > max) {
		m_servo = max;
	}

	m_tiltSrv.Set(m_servo);
	return (m_servo);
}

float WeejusCamera::getAngle() {
	return m_tiltSrv.Get() * SERVO_RANGE - SERVO_LOCAL_DELTA_ANGLE;
}

HSLImage* WeejusCamera::getImage() {
	return NULL;
}

char* WeejusCamera::getDebugLineCamera(int line) {
	switch (line) {
	case 1:
		snprintf(_dbgBuf, sizeof(_dbgBuf), "servo: %1.3f (%1.3f)",
				(double) m_servo, (double) m_tiltSrv.Get());
		break;
	case 2:
		snprintf(_dbgBuf, sizeof(_dbgBuf), "Flp: %d   Lght: %d",
				(int) m_globalFlipped, (m_lightRing.Get() == Relay::kOn));
		break;
	case 3:
		snprintf(_dbgBuf, sizeof(_dbgBuf), "");
		break;
	default:
		snprintf(_dbgBuf, sizeof(_dbgBuf), "");
		break;
	}
	return _dbgBuf;
}
