#include "WeejusClimberDrive.h"
#include "IOMap.h"

#include "WeejusConfig.h"

// Medium speed;
const float UNLOAD_SEEK_POWER = 0.4;

// .5 volts is 1 rotation; ~2.66 V over entire path
const float POT_LOCATION_EQUALITY_INTERVAL_RADIUS = 0.125;//volts

//TODO calculate a first approximation
const float MAX_SHUTTLE_SPEED = 1.0;// Pot Volts/sec

// NOTE: these only apply IFF USING_POT_LIMITS (see header) is true;
// TODO measure & calculate these relatively

// measured 18th Feb
const float POT_HIGH_LIMIT = 3.14; // Pot Volts

// defined 18th Feb
const float POT_MIDDLE_LIMIT = 1.4; // Pot Volts

// measured 18th Feb
const float POT_LOW_LIMIT = 0.48; // Pot Volts

//const float POT_INTERLIMIT_RANGE = POT_HIGH_LIMIT - POT_LOW_LIMIT;

//const float SLOWDOWN_LENGTH = 0.04; // Volts

const float MAX_CURRENT = 37.0;// cap at 37 AMPS

/*
 * TODO:
 * 
 *
 * Transform limits into Counters && readouts
 * 
 * Order the functions and use |* SEPERATORS *| 
 * 
 */

/*
 * is the first arg within third arg of second arg
 */
inline bool inInterval(float val, float int_c, float int_r) {
	return ((val >= int_c - int_r) && (val <= int_c + int_r));
}

// TODO unfunctionalize
inline float normalize(float in) {
	if (in > 1)
		return 1;
	if (in < -1)
		return -1;
	return in;
}

WeejusClimberDrive::WeejusClimberDrive() {

	top_limit = new DigitalInput(DIG_CLIMB_SHUTTLE_MAX_EXTENSION);
	bottom_limit = new DigitalInput(DIG_CLIMB_SHUTTLE_MAX_RETRACTION);
	middle_limit = new DigitalInput(DIG_CLIMB_SHUTTLE_DEPLOY_POSITION);

	arms = new WeejusCANJaguar(CANID_CLIMBER_RIGHT,
			WeejusCANJaguar::kPercentVbus);

	pot_enc = new WeejusPotEncoder(ANA_CLIMB_LIFT_ARMS_POT, MAX_SHUTTLE_SPEED);

	// set to false by default
	limits_broken = false;

	// set to true by default;
	// corresponds to BROKEN Climber default;
	enabled = true;
}

void WeejusClimberDrive::initialize() {
	arms->Set(0);
	output_power = 0;

	shuttle_region = UNKNOWN;
	homing = false;

	hit_top_limit = false;
	hit_bottom_limit = false;
	under_center_limit = false;
}

WeejusClimberDrive::~WeejusClimberDrive() {
	delete top_limit;
	delete middle_limit;
	delete bottom_limit;
	delete pot_enc;
	delete arms;
}

void WeejusClimberDrive::setLimitsBroken(bool broken) {
	if (broken == limits_broken)
		return;
	limits_broken = broken;

	if (limits_broken) {
		DBG_PRINTF(DBG_CLIMBER_DRIVE, "[Climber Drive] Marking pot broken\n");
	} else {
		DBG_PRINTF(DBG_CLIMBER_DRIVE, "[Climber Drive] Marking pot functional\n");
	}
}

bool WeejusClimberDrive::checkUpperLimit() {
	if (limits_broken) {
		hit_top_limit = false;
		return false;
	}

	bool val = getTopLimit();
	if (val) {
		DBG_PRINTF(DBG_CLIMBER_DRIVE_CONT, "[Climber Drive] Top limit hit & set\n");
		hit_top_limit = true;
	}
	return hit_top_limit;
}

bool WeejusClimberDrive::checkLowerLimit() {
	if (limits_broken) {
		hit_bottom_limit = false;
		return false;
	}

	bool val = getBottomLimit();
	if (val) {
		DBG_PRINTF(DBG_CLIMBER_DRIVE_CONT, "[Climber Drive] Bottom limit hit & set\n");
		hit_bottom_limit = true;
	}
	return hit_bottom_limit;
}

// TODO check logic; may be incomplete
void WeejusClimberDrive::processHoming() {
	if (getMiddleLimit()) {
		DBG_PRINTF(DBG_CLIMBER_DRIVE, "[Climber Drive] processHoming: hit center, finished\n");
		homing = false;
		shuttle_region = CENTER;
		quickStop();
		return;
	}

	if (checkUpperLimit()) {
		DBG_PRINTF(DBG_CLIMBER_DRIVE, "[Climber Drive] processHoming: hit top, marked UPPER, returning to middle\n");
		shuttle_region = UPPER;
		setPower(-UNLOAD_SEEK_POWER);
		// homing remains true
	}
}

// CIMS are chained together 
void WeejusClimberDrive::process() {
	if (!enabled) {
		DBG_PRINTF(DBG_CLIMBER_DRIVE_CONT, "[Climber Drive] disabled, not processing\n");
		return;
	}

	if (homing) {
		processHoming();
	}

	DBG_PRINTF(DBG_CLIMBER_DRIVE_CONT, "[Climber Drive] Main loop just before checklimits\n");
	checkUpperLimit();
	checkLowerLimit();
	setSafeSpeed();
}

/*
 * Just stop it where it is
 */
void WeejusClimberDrive::quickStop() {
	DBG_PRINTF(DBG_CLIMBER_DRIVE, "[Climber Drive] quickStop\n");
	output_power = 0;
	arms->Set(0);
}

// TODO incomplete implementation 
// FIX before use
void WeejusClimberDrive::goToMiddle() {
	DBG_PRINTF(DBG_CLIMBER_DRIVE, "[Climber Drive] goToMiddle() called\n");

	if (shuttle_region == UNKNOWN) {
		if (USING_POT_LIMITS && !limits_broken) {
			if (inInterval(pot_enc->GetPos(), POT_MIDDLE_LIMIT,
					POT_LOCATION_EQUALITY_INTERVAL_RADIUS)) {
				DBG_PRINTF(DBG_CLIMBER_DRIVE, "[Climber Drive] pot limits yielded CENTER\n");
				shuttle_region = CENTER;
			} else if (pot_enc->GetPos() > POT_MIDDLE_LIMIT) {
				DBG_PRINTF(DBG_CLIMBER_DRIVE, "[Climber Drive] pot limits yielded UPPER\n");
				shuttle_region = UPPER;
			} else {
				DBG_PRINTF(DBG_CLIMBER_DRIVE, "[Climber Drive] pot limits yielded LOWER\n");
				shuttle_region = LOWER;
			}
		} else if (getMiddleLimit()) {
			DBG_PRINTF(DBG_CLIMBER_DRIVE, "[Climber Drive] REAL limit yielded CENTER\n");
			shuttle_region = CENTER;
		} else if (getTopLimit()) {
			DBG_PRINTF(DBG_CLIMBER_DRIVE, "[Climber Drive] REAL limit yielded UPPER\n");
			shuttle_region = UPPER;
		} else if (getBottomLimit()) {
			DBG_PRINTF(DBG_CLIMBER_DRIVE, "[Climber Drive] REAL limit yielded LOWER\n");
			shuttle_region = LOWER;
		} else {

			DBG_PRINTF(DBG_CLIMBER_DRIVE, "[Climber Drive] moving up until the next limit\n");
			homing = true;
			// go up to the nearest limit
			setPower(UNLOAD_SEEK_POWER);
			DBG_PRINTF(DBG_CLIMBER_DRIVE, "[Climber Drive] the power is now %f\n", output_power);
			return;
		}
	}

	if (shuttle_region == CENTER) {
		DBG_PRINTF(DBG_CLIMBER_DRIVE, "[Climber Drive] already at center\n");
		return;
	} else if (shuttle_region == UPPER) {
		DBG_PRINTF(DBG_CLIMBER_DRIVE, "[Climber Drive] at upper, moving down\n");
		setPower(-UNLOAD_SEEK_POWER);
		homing = true;
		return;
	} else if (shuttle_region == LOWER) {
		DBG_PRINTF(DBG_CLIMBER_DRIVE, "[Climber Drive] at lower, moving up\n");
		setPower(UNLOAD_SEEK_POWER);
		homing = true;
		return;
	}
}

/* * * * * * * * * * * POWER * * * * * * * * * * * * */

void WeejusClimberDrive::setPower(float power) {
	output_power = normalize(power);

	checkUpperLimit();
	checkLowerLimit();
	setSafeSpeed();
}

void WeejusClimberDrive::setSafeSpeed() {
	DBG_PRINTF(DBG_CLIMBER_DRIVE_CONT, "[Climber Drive] Output power starts at %f\n", output_power);

	// check for hit top/bottom limits
	DBG_PRINTF(DBG_CLIMBER_DRIVE_CONT, "[Climber Drive] Limits: H: %d L: %d\n", hit_top_limit, hit_bottom_limit);
	if (hit_top_limit) {
		DBG_PRINTF(DBG_CLIMBER_DRIVE_CONT, "[Climber Drive] Adjusting for top limit\n");
		if (output_power > 0) {
			DBG_PRINTF(DBG_CLIMBER_DRIVE_CONT, "[Climber Drive] Cutting power\n");
			output_power = 0;
		} else {
			hit_top_limit = getTopLimit();
		}
	}

	if (hit_bottom_limit) {
		DBG_PRINTF(DBG_CLIMBER_DRIVE_CONT, "[Climber Drive] Adjusting for bottom limit\n");
		if (output_power < 0) {
			DBG_PRINTF(DBG_CLIMBER_DRIVE_CONT, "[Climber Drive] Cutting power\n");
			output_power = 0;
		} else {
			hit_bottom_limit = getBottomLimit();
		}
	}

	DBG_PRINTF(DBG_CLIMBER_DRIVE_CONT, "[Climber Drive] Output power ends at %f \n", output_power);
	arms->Set(output_power);
}

float WeejusClimberDrive::getPower() {
	return output_power;
}

/* * * * * * * * * * * ENABLE/DISABLE * * * * * * * * */

void WeejusClimberDrive::disable() {
	DBG_PRINTF(DBG_CLIMBER_DRIVE, "[Climber Drive] disabled\n");
	enabled = false;

	//
	// This does not quickStop; that would stop
	// homing actions by resetting output power
	//
	arms->Set(0);
}

void WeejusClimberDrive::enable() {
	DBG_PRINTF(DBG_CLIMBER_DRIVE, "[Climber Drive] enabling\n");
	enabled = true;
}

bool WeejusClimberDrive::atMiddle() {
	return getMiddleLimit();
}

bool WeejusClimberDrive::wasTopLimitHit() {
	return hit_top_limit;
}
bool WeejusClimberDrive::wasBottomLimitHit() {
	return hit_bottom_limit;
}

double WeejusClimberDrive::getSpeed() {
	return pot_enc->ReadVelocity();
}
double WeejusClimberDrive::getPos() {
	return pot_enc->GetPos();
}

/*  g->Get() is False corresponds to a blocked limit.
 * Therefore, we invert values to be true when blocked,
 * false when clear
 */
bool WeejusClimberDrive::getTopLimit() {
	return !top_limit->Get();
}

bool WeejusClimberDrive::getMiddleLimit() {
	return !middle_limit->Get();
}

bool WeejusClimberDrive::getBottomLimit() {
	return !bottom_limit->Get();
}

float WeejusClimberDrive::getAvgCurrent() {
	return arms->GetOutputCurrent();
}
