#include "WeejusPotEncoder.h"

WeejusPotEncoder::WeejusPotEncoder(UINT32 channel, float max_speed) :
	m_semaphore(0) {

	m_semaphore = semBCreate(SEM_Q_PRIORITY, SEM_FULL);
	m_controlLoop = new Notifier(WeejusPotEncoder::CallUpdateVelocity, this);

	m_pot = new AnalogChannel(channel);
	//setting the details

	// default is 8000 Hz
	// we shall never go below 200Hz
	// so sample 8 times, w/oversample 16
	m_pot->SetAverageBits(3);
	m_pot->SetOversampleBits(1);
	m_max_speed = max_speed;

	// remember; there is a whole host of configuration options
	// available for the pot
	m_last_position = m_pot->GetAverageVoltage();
	m_velocity = 0;
	m_last_time = GetTime();// supposedly nanosec res.

	m_controlLoop->StartPeriodic(pot_update_period);
}

// See the WeejusPIDController's CallCalculate 
void WeejusPotEncoder::CallUpdateVelocity(void *controller) {
	WeejusPotEncoder* control = (WeejusPotEncoder*) controller;
	control->UpdateVelocity();
}

/*
 * Only called indirectly by CallUpdateVelocity
 * 
 */
void WeejusPotEncoder::UpdateVelocity() {
	// it seems almost everything is done 
	// internally
	CRITICAL_REGION(m_semaphore)
		{

			double new_pos, new_time;
			// since we only read our pot
			// it doesn't matter where
			// we put it

			// update and calculate velocity
			new_pos = m_pot->GetAverageVoltage();
			new_time = GetTime();
			m_velocity = (new_pos - m_last_position) / (new_time - m_last_time)
					/ m_max_speed;
			m_last_position = new_pos;
			m_last_time = new_time;

		}END_REGION;
}

double WeejusPotEncoder::PIDGet() {
	double vel;

	CRITICAL_REGION(m_semaphore)
		{
			vel = m_velocity;
		}END_REGION;

	return vel;
}

double WeejusPotEncoder::GetPos() {
	double pos;
	CRITICAL_REGION(m_semaphore)
		{
			pos = m_last_position;
		}END_REGION;

	return pos;
}

double WeejusPotEncoder::ReadVelocity() {
	double vel;

	CRITICAL_REGION(m_semaphore)
		{
			vel = m_velocity;
		}END_REGION;

	return vel;
}

WeejusPotEncoder::~WeejusPotEncoder() {
	semFlush(m_semaphore);
	delete m_pot;
	delete m_controlLoop;
}

void WeejusPotEncoder::Disable() {

}

void WeejusPotEncoder::Enable() {

}
