#ifndef _TARGET_H_
#define _TARGET_H_
#include  "WPILib.h"
#include "WeejusCamera.h"
//#include "WeejusVision.h"

class WeejusTarget: public WeejusCamera {
public:

	/*
	 * Method: getInstance();
	 * 
	 * Inputs: Null
	 * Outputs: pointer to the only WeejusTarget object
	 * 
	 * Remarks:
	 * Returns a pointer to the only instance of the WeejusTarget
	 * If the instance has not been created, it will be, otherwise the existing instance will be returned
	 */
	static WeejusTarget* getInstance();

	//DO NOT CALL THIS
	~WeejusTarget();

	/*
	 * Method: getCalcDistance();
	 * 
	 * Inputs: Null
	 * Outputs: float representing the calculated distance in inches
	 * 
	 * Remarks:
	 * returns the distance previously calculated by findTargets()
	 * distance may not be exact, but accurate enough
	 * will return -1 until findTargets() has sucessfully found and processed a target, or a target was not found
	 * otherwise, will return the last distance calculated by findTargets()
	 */
	float getCalcDistance();

	/*
	 * Method: getCalcAngle();
	 * 
	 * Inputs: Null
	 * Outputs: float representing the calculated angle in degrees
	 * 
	 * Remarks:
	 * returns the calculated angle that would aim the shooter at the target
	 * angle previously calculated by findTargets()
	 * will return -1 until findTargets() has sucessfully found and processed a target, or a target was not found
	 * otherwise, will return the last angle calculated by findTargets()
	 */
	float getCalcAngle();

	/*
	 * Method: findTargets();
	 * 
	 * Inputs: Null
	 * Outputs: Null
	 * 
	 * Remarks:
	 * gets the image from the camera, finds targets in image, and calculates distance and angle
	 * will calculate based on target found closest to center of the image
	 * MUST be called before calling getCalcDistance() or getCalcAngle()
	 */
	void findTargets();

	/*
	 * Method: getDebugLine(int);
	 * 
	 * Inputs:
	 * 			int line: integer representing the debug line to get (range 1->6)
	 * Outputs: char array containing the debug line to print
	 * 
	 * Remarks:
	 * will return the line specified by the number passed to the parameter
	 * only can return 6 debug lines
	 * if line < 1, will return line 1
	 * if line > 6, will return line 6
	 */
	char* getDebugLine(int line);

private:
	static WeejusTarget* theTarget;
};
#endif
